/* file: annpan.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions for the annotations panel of WAVE

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include "wave.h"
#include "gwave.h"
#include "interface.h"
#include "support.h"

GtkWidget *ann_frame;
GtkWidget *anntyp_item, *subtyp_item, *num_item, *chan_item, *aux_item;

int ann_popup_active = -1;

/* Make the annotation template popup window disappear. */
void
dismiss_ann_template(GtkButton *button, gpointer user_data)
{
  if (ann_popup_active > 0) {
    gtk_widget_hide(ann_frame);
    ann_popup_active = 0;
  }
}


gboolean
on_annotation_template_delete_event(GtkWidget *widget,
				    GdkEvent *event, gpointer user_data)
{
  dismiss_ann_template(NULL, NULL);
  return FALSE;
}



/* Update the annotation template following a selection in the popup window. */
void
ann_select(GtkWidget *widget, gpointer user_data)
{
  int a;
  static char tmp_aux[257];
  
  switch ( (a = GPOINTER_TO_INT(user_data)) ) {
  case 't': ann_template.anntyp = get_menu_active_index( GTK_MENU(widget) ); break;
  case 's': ann_template.subtyp = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(widget) ); break;
  case 'n': ann_template.num    = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(widget) ); break;
  case 'c': ann_template.chan   = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(widget) ); break;
  case 'a': strcpy(tmp_aux+1, (char *)gtk_entry_get_text( GTK_ENTRY(widget)) );
    if ( (tmp_aux[0] = strlen(tmp_aux+1)) )
      ann_template.aux = tmp_aux;
    else
      ann_template.aux = NULL;
    break;
  }
}

static char *mstr[54];

static void create_mstr_array()
{
  char *dp, mbuf[5], *mp;
  int i;
  unsigned int l, lm;

  mstr[0] = ".    (Deleted annotation)";
  for (i = 1; i <= ACMAX; i++) {
    if ((mp = annstr(i)) == NULL) {
      sprintf(mbuf, "[%d]", i);
      mp = mbuf;
    }
    if ((dp = anndesc(i)) == NULL)
      dp = "(unassigned annotation type)";
    lm = strlen(mp);
    l = strlen(dp) + 6;
    if (lm > 4) l += lm - 4;
    if ((mstr[i] = (char *)malloc(l)) == NULL)
      break;
    strcpy(mstr[i], mp);
    do {
      mstr[i][lm] = ' ';
    } while (++lm < 5);
    strcpy(mstr[i]+lm, dp);
  }
  mstr[ACMAX+1] = ":    (Index mark)";
  mstr[ACMAX+2] = "<    (Start of analysis)";
  mstr[ACMAX+3] = ">    (End of analysis)";
  mstr[ACMAX+4] = ";    (Reference mark)";
}

/* Set up popup window for defining an annotation template. */
static void create_ann_template_popup()
{
  int i;
  GtkWidget *menu, *menuitem;

  create_mstr_array();
  ann_template.anntyp = 1;

  ann_frame = create_annotation_template ();

  anntyp_item = lookup_widget( GTK_WIDGET( ann_frame ), "anntyp_option");
  menu = gtk_menu_new();
  for (i=0; i<54; i++) {
    menuitem = gtk_radio_menu_item_new_with_label(NULL, mstr[i]);
    gtk_menu_append( GTK_MENU( menu ), menuitem);
    gtk_widget_show(menuitem);
  }
  gtk_signal_connect( GTK_OBJECT (menu), "selection-done", 
		      GTK_SIGNAL_FUNC(ann_select), GINT_TO_POINTER((gint)'t'));
  gtk_option_menu_remove_menu( GTK_OPTION_MENU( anntyp_item ) );
  gtk_option_menu_set_menu( GTK_OPTION_MENU( anntyp_item ), menu );
  gtk_option_menu_set_history( GTK_OPTION_MENU( anntyp_item ), 1 );

  aux_item = lookup_widget( GTK_WIDGET( ann_frame ), "aux_entry" );
  subtyp_item = lookup_widget( GTK_WIDGET( ann_frame ), "subtype_spinbutton");
  chan_item = lookup_widget( GTK_WIDGET( ann_frame ), "chan_spinbutton");
  num_item = lookup_widget( GTK_WIDGET( ann_frame ), "num_spinbutton");

}

/* Make the annotation template popup window appear. */
void show_ann_template()
{
  if (ann_popup_active < 0) create_ann_template_popup();
  gtk_widget_show( ann_frame );
  ann_popup_active = 1;
}

/* Set the annotation template window `Type' item. */
void set_anntyp(i)
int i;
{
  if (ann_popup_active < 0) create_ann_template_popup();
  gtk_option_menu_set_history ( GTK_OPTION_MENU( anntyp_item ), i );
}

/* Set the annotation template window `Aux' item. */
void set_ann_aux(s)
char *s;
{
  if (ann_popup_active >= 0)
    gtk_entry_set_text( GTK_ENTRY( aux_item ), s ? s+1 : "");
}

/* Set the annotation template window `Subtype' item. */
void set_ann_subtyp(i)
int i;
{
  if (ann_popup_active >= 0)
    gtk_spin_button_set_value( GTK_SPIN_BUTTON( subtyp_item ), i );
}

/* Set the annotation template window `Chan' item. */
void set_ann_chan(i)
int i;
{
  if (ann_popup_active >= 0)
    gtk_spin_button_set_value( GTK_SPIN_BUTTON( chan_item ), i );
}

/* Set the annotation template window `Num' item. */
void set_ann_num(i)
int i;
{
  if (ann_popup_active >= 0)
    gtk_spin_button_set_value( GTK_SPIN_BUTTON( num_item ), i );
}
