/* file: console.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions for the console panel of Wave. Contains code for linux
and Windows.

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include "wave.h"
#include "gwave.h"
#include "defaults.h"
#include "interface.h"
#include "support.h"
#include <stdio.h>

#ifdef __WIN32__
#include <windows.h>

HANDLE __win32api__console_create();
void __win32api__console_input(HANDLE console_out, gchar *cmd, gint len);

HANDLE console_create()
{
  return __win32api__console_create();
}

void console_input(HANDLE console_out, gchar *cmd, gint len) {
  __win32api__console_input(console_out, cmd, len);
}

HANDLE __win32api__console_create() {

  SECURITY_ATTRIBUTES sa;
  BOOL result; 
  STARTUPINFO si;
  PROCESS_INFORMATION pi; 
  LPSTR cmd_shell;
  HANDLE pipe_stdin_rd, pipe_stdin_wr, pipe_stdin_wr_cpy,
    con_stdin, con_stdout; 
  //FILE * debug;
  gchar os_ver = get_os();

  if ( os_ver & OS_WIN95 )
    cmd_shell = defaults_get_string( "Shell", "Win95Cmd.exe" );
  else if( os_ver & OS_WINNT )
    cmd_shell = defaults_get_string( "Shell", "C:\\WINNT\\system32\\CMD.EXE" );
  else
    cmd_shell = defaults_get_string( "Shell", NULL );

  /*
  debug = fopen("debug.txt","a");
  fprintf(debug, "info: cmd shell = %s\n", cmd_shell);
  fclose( debug );
  */

  if ( cmd_shell == NULL )
    return NULL;

  AllocConsole();
  sa.nLength = sizeof(SECURITY_ATTRIBUTES);
  sa.bInheritHandle = TRUE;
  sa.lpSecurityDescriptor = NULL;  

  /* create console process. */
  ZeroMemory( &pi, sizeof(PROCESS_INFORMATION) );
  ZeroMemory( &si, sizeof(STARTUPINFO) );
  si.cb = sizeof(STARTUPINFO); 
  
  if ( ! CreateProcess( cmd_shell, cmd_shell, NULL, NULL, TRUE,
  			0, NULL, NULL, &si, &pi) ) 
    exit(0);
  
  return GetStdHandle(STD_INPUT_HANDLE);

}

void
__win32api__console_input(HANDLE console_out, gchar *cmd, gint len) {
  gint i;
  DWORD dwWritten; 
  INPUT_RECORD *input_buf;
  input_buf = g_malloc( 2 * len * sizeof(INPUT_RECORD) );

  for ( i=0; i<len; i++ ) {
    input_buf[2*i].Event.KeyEvent.bKeyDown = TRUE; 
    input_buf[2*i+1].Event.KeyEvent.bKeyDown = FALSE; 
    input_buf[2*i].Event.KeyEvent.wRepeatCount 
      = input_buf[2*i+1].Event.KeyEvent.wRepeatCount = 1; 
    if ( cmd[i] == '\n' )
      input_buf[2*i].Event.KeyEvent.uChar.UnicodeChar 
	= input_buf[2*i+1].Event.KeyEvent.uChar.UnicodeChar = 13;
    else
      input_buf[2*i].Event.KeyEvent.uChar.AsciiChar 
	= input_buf[2*i+1].Event.KeyEvent.uChar.AsciiChar = cmd[i];
    input_buf[2*i].Event.KeyEvent.wVirtualKeyCode 
      = input_buf[2*i+1].Event.KeyEvent.wVirtualKeyCode = 
      VkKeyScan( input_buf[2*i].Event.KeyEvent.uChar.AsciiChar );
    input_buf[2*i].Event.KeyEvent.wVirtualScanCode 
      = input_buf[2*i+1].Event.KeyEvent.wVirtualScanCode = 
      MapVirtualKey( input_buf[2*i].Event.KeyEvent.wVirtualKeyCode, 0);
    input_buf[2*i].Event.KeyEvent.dwControlKeyState 
      = input_buf[2*i+1].Event.KeyEvent.dwControlKeyState = 0;
    input_buf[2*i].EventType = input_buf[2*i+1].EventType = KEY_EVENT;    
  }

  WriteConsoleInput(console_out, input_buf, 2*len, &dwWritten);  
  
  g_free( input_buf );
}

/* Make the analysis command window appear. */
void
show_command_window(GtkButton *button,
		    gpointer user_data)
{
  
}

gboolean on_analysis_cmds_delete_event(GtkWidget *widget, 
				       GdkEvent *event, gpointer user_data)
{
  console_input(console, "exit", 6);
  return TRUE;
}

void dismiss_analysis_cmds(GtkButton *button, gpointer user_data)
{
  console_input(console, "exit", 6);
}

#else
#include <zvt/zvtterm.h>
#include <pwd.h>

GtkWidget* __zvt__console_create();
void __zvt__console_input(GtkWidget *term, gchar *cmd, gint len);

GtkWidget *console_create()
{
  return __zvt__console_create();
}

void console_input(GtkWidget *term, gchar *cmd, gint len)
{
  __zvt__console_input(term, cmd, len);
}

GtkWidget *console_frame;
extern char **environ;             
static char **env;
static char **env_copy;
static int winid_pos;

GtkWidget *
__zvt__console_create() {
  gint i;
  char **p;
  struct passwd *pw;
  GString *shell, *name;
  GtkWidget *con, *box, *scroll;
  
  /* set up terminal environment */
  env = environ;
  
  for (p = env; *p; p++);
  i = p - env;
  env_copy = (char **) g_malloc (sizeof (char *) * (i + 3));
  for (i = 0, p = env; *p; p++) {
    if (strncmp (*p, "TERM=", 5) == 0) {
      env_copy [i++] = "TERM=xterm";
    } else if ((strncmp (*p, "COLUMNS=", 8) == 0) ||
	       (strncmp (*p, "LINES=", 6) == 0)) {
      continue;
    } else {
      env_copy [i++] = *p;
    }
  }
  
  env_copy [i++] = "COLORTERM=zterm";
  winid_pos = i++;
  env_copy [winid_pos] = "TEST";
  env_copy [i] = NULL;
  
  console_frame = create_analysis_cmds();
  box = lookup_widget( GTK_WIDGET(console_frame), "analysis_cmds_frame" );
  scroll = lookup_widget( GTK_WIDGET(console_frame), "analysis_cmds_vscrollbar");
  
  con = zvt_term_new();
  zvt_term_set_scrollback( ZVT_TERM (con), 100);
  zvt_term_set_scroll_on_output( ZVT_TERM (con), TRUE );
  zvt_term_set_scroll_on_keystroke( ZVT_TERM (con), TRUE );
  zvt_term_set_background (ZVT_TERM (con), NULL, 0, 0);
  zvt_term_set_size(ZVT_TERM (con), 60, 10);
  
  gtk_range_set_adjustment( GTK_RANGE (scroll), 
			    GTK_ADJUSTMENT ((ZVT_TERM (con)->adjustment)) );
  GTK_WIDGET_UNSET_FLAGS (scroll, GTK_CAN_FOCUS);
  
  gtk_container_add( GTK_CONTAINER (box), GTK_WIDGET (con) );
  gtk_widget_show( GTK_WIDGET(con) );
  //gtk_widget_show( GTK_WIDGET(console_frame) );
  
  switch (zvt_term_forkpty(ZVT_TERM (con), 
			   ZVT_TERM_DO_UTMP_LOG |  ZVT_TERM_DO_WTMP_LOG)) {
  case -1:
    perror("ERROR: unable to fork:");
    exit(1);
    break;
  case 0:           
    /* get shell from passwd */
    pw = getpwuid(getuid());
    if (pw) {
      shell = g_string_new(pw->pw_shell);
      name = g_string_new(""); 
      g_string_append(name, strrchr(pw->pw_shell, '/'));
    } 
    else {
      shell = g_string_new("/bin/sh");
      name = g_string_new("sh");
    }
    execle (shell->str, name->str, NULL, env_copy);
    perror("ERROR: Cannot exec command:");
    exit(1);
    
  default:
    break;
  }  
  return con;
}


void __zvt__console_input(GtkWidget *term, gchar *cmd, gint len) {
  write( (ZVT_TERM (term))->vx->vt.keyfd, cmd, len);
}

/* Make the analysis command window appear. */
void
show_command_window(GtkButton *button,
		    gpointer user_data)
{
  gtk_widget_show(console_frame);
  //if ( GTK_WIDGET_CAN_FOCUS (console) && ! GTK_WIDGET_HAS_FOCUS(console) )
  //  gtk_widget_grab_focus( GTK_WIDGET(console) );
}

gboolean on_analysis_cmds_delete_event(GtkWidget *widget, 
				       GdkEvent *event, gpointer user_data)
{
  dismiss_analysis_cmds(NULL,NULL);
  return TRUE;
}

void dismiss_analysis_cmds(GtkButton *button, gpointer user_data)
{
  gtk_widget_hide(console_frame);
}

#endif





