/* file: edit.c	        I. Henry        20 July 2001
			Last revised:   20 July 2001
Various support functions 

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include "wave.h"
#include "gwave.h"
#include "interface.h"
#include "support.h"
#include <gdk/gdkprivate.h>

#ifdef __WIN32__
#include <windows.h>
#include <gdk/win32/gdkwin32.h>
void
pointer_warp( gint dest_x, gint dest_y)
{
  GdkWindow *src_w;
  GdkWindowPrivate *src_private;

  src_w = gtk_widget_get_parent_window( canvas );
  src_private = (GdkWindowPrivate *) src_w;

  SetCursorPos ( src_private->x + dest_x, 
		 src_private->y + dest_y );
}


#else
#include <gdk/gdkx.h>

void
pointer_warp( gint dest_x, gint dest_y)
{
  GdkWindow *src_w;
  GdkWindowPrivate *src_private;
  Window xsrc_w;

  src_w = gtk_widget_get_parent_window( canvas );
  src_private = (GdkWindowPrivate *) src_w;

  if (src_private)
    xsrc_w = src_private->xwindow;
  else
    xsrc_w = None;


  XWarpPointer (gdk_display,
                None, xsrc_w,  
                0, 0, 0, 0,
                canvas->allocation.x + dest_x, canvas->allocation.y + dest_y);
}

#endif

gint get_option_menu_active_index( GtkOptionMenu *option_menu ) {
  GtkWidget *menu, *active_item;
  menu = gtk_option_menu_get_menu(option_menu);
  active_item = gtk_menu_get_active (GTK_MENU (menu)); 
  return g_list_index (GTK_MENU_SHELL (menu)->children, active_item);
}

gint get_menu_active_index( GtkMenu *menu ) {
  GtkWidget *active_item;
  active_item = gtk_menu_get_active (GTK_MENU (menu)); 
  return g_list_index (GTK_MENU_SHELL (menu)->children, active_item);
}


/* mkstemp is not in mingw, so make one using mktemp */
#ifdef __WIN32__

int mkstemp(char *template) {
  if ( mktemp( template ) == NULL )
    return -1;
  else
    /* this should return the file descriptor, not 0 */
    return 0;
}

#endif

gchar get_os() {

  gchar os_ver;

#ifdef __WIN32__
  /* this is a windows system, get the Windows version. */
  
  DWORD winver = GetVersion();
  DWORD major_version =  (DWORD)(LOBYTE(LOWORD(winver)));

  if (winver < 0x80000000)
    os_ver = OS_WINNT;
  else if (major_version < 4)
    os_ver = OS_MS_UNKNOWN;
  else
    os_ver = OS_WIN95;

#else /* this is unix system */
   /* fill in other Unixes as gtk wave is ported */
   os_ver = OS_LINUX;

#endif

   return os_ver; 

}


