/* file: wave.h	        I. Henry        20 July 2001
			Last revised:   20 July 2001
The include file drawing and GTK+ functions.

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#ifdef __WIN32__
#include <windows.h>
#endif
#include <gtk/gtk.h>

/* Default screen resolution.  All known X servers seem to know something about
   the screen size, even if what they know isn't true.  Just in case, these
   constants define default values to be used if Height/WidthMMOfScreen give
   non-positive results.  If your screen doesn't have square pixels, define
   DPMMX and DPMMY differently.  Note that these definitions can be made using
   -D... options on the C compiler's command line.

   If your X server is misinformed about your screen size, there are three
   possible remedies.  All of the X11R4 sample servers from MIT accept a `-dpi'
   option that allows you to override the server default at the time the
   server is started.  See the X man page for details.  Alternatively, use
   WAVE's `-dpi' option or `dpi' resource (less desirable solutions, since they
   don't solve similar problems that other applications are likely to
   encounter). */
#ifndef DPMM
#define DPMM	4.0	/* default screen resolution (pixels per millimeter) */
#endif
#ifndef DPMMX
#define DPMMX	DPMM
#endif
#ifndef DPMMY
#define DPMMY	DPMM
#endif

/* Atnnotations are displayed in the font specified by DEFANNFONT if no
   other choice is specified.  At present, there is no other way to specify
   a choice.  Ultimately, this should be done by reading a resource file. */
#ifndef DEFANNFONT
#define DEFANNFONT	"fixed"
#endif

#define NUM_L     5
#define GRID_L    0
#define ANNOT_L   1
#define SIGNAL_L  2
#define HSIGNAL_L 3
#define CURSOR_L  4

#define WAVE_DEFAULTS_FILE 0x01
#define WAVE_ANNTAB_FILE   0x02
#define WAVE_MENU_FILE     0x04
#define WAVE_FILE_ENV      0x08
#define WAVE_FILE_DEF      0x10
#define WAVE_FILE_USER     0x20
#define WAVE_FILE_GLOBAL   0x40
#define WAVE_FILE_EDITABLE 0x80
#define WAVE_FILE_BEST     0x78

struct Wave_layer {
  GdkBitmap *bitmap;  /* Bitmap                                  */
  GdkGC *draw;        /* GC used to draw to bitmap               */
  GdkGC *draw2;       /* GC used to draw to bitmap               */
  GdkGC *clear;       /* GC used to clear bitmap                 */
  GdkGC *clear2;      /* GC used to clear bitmap                 */
};

typedef struct Wave_layer Wave_Layer;

#ifdef INIT
#define COMMON
#else
#define COMMON extern
#endif

/* Globally visible GTK objects. */
COMMON GtkWidget *gtk_wave;        /* pointer to main window widget */
COMMON GtkWidget *canvas;          /* pointer to the GtkDrawarea */ 
COMMON GdkPixmap *backend_pixmap;  /* backend gdk pixmap */
COMMON struct footer { GtkWidget *right; GtkWidget *left;} footer;
COMMON GdkFont *font;	           /* Font used for text in signal window */
COMMON GdkCursor *cursor;          /* Cursor for canvas */
COMMON GdkGC *bitmap_clear, *bitmap_set, *bitmap_gh_set, *bg_fill;

/* layers */
COMMON Wave_Layer layer[NUM_L];

COMMON int ann_popup_active;	/* <0: annotation template popup not created,
				    0: popup not visible, >0: popup visible */

#ifdef __WIN32__
COMMON HANDLE *console;
#else
COMMON GtkWidget *console;
#endif

/* Display lists

A display list contains all information needed to draw a screenful of signals.
For each of the nsig signals, a display list contains a list of the
(x, y) pixel coordinate pairs that specify the vertices of the polyline
that represents the signal.  

A cache of recently-used display lists (up to MAX_DISPLAY_LISTS of them) is
maintained as a singly-linked list;  the first display list is pointed
to by first_list.
*/

struct display_list {
  struct display_list *next;	 /* link to next display list */
  long start;		         /* time of first sample */
  int nsig;		         /* number of signals */
  int npoints;	                 /* number of (input) points per signal */
  int ndpts;		         /* number of (output) points per signal */
  int xmax;		         /* largest x, expressed as window abscissa */
  int sb[WFDB_MAXSIG];           /* signal baselines, expressed as window ordinates */
  GdkPoint *vlist[WFDB_MAXSIG];  /* vertex list pointers for each signal */
};
COMMON struct display_list *first_list;

/* Function prototypes for ANSI C and C++ compilers.  (Most of these are in
   wave.h;  those below are only for functions that use X or XView objects
   in their argument lists.) */

extern void create_main_panel();	                      /* in mainpan.c  */
extern void disp_proc(GtkWidget *widget, gpointer user_data); /* in mainpan.c  */
extern gboolean window_event_proc(GtkWidget *widget,          /* in edit.c */
				  GdkEvent *event, gpointer user_data);

extern gboolean key_event_proc(GtkWidget *widget, GdkEventKey *event, gpointer user_data);

extern struct display_list *find_display_list(long time); /* in signal.c */

/* added for gtk-wave */
extern int notice_prompt();                                                      /* in notice.c */
extern gint get_option_menu_active_index( GtkOptionMenu *option_menu );          /* in extra.c */
extern gint get_menu_active_index( GtkMenu *menu );                              /* in extra.c */
extern void pointer_warp( gint dest_x, gint dest_y);                             /* in extra.c */
extern void show_search_template(GtkButton *button, gpointer user_data);         /* in search.c */
extern void on_analyze_menu_activate(GtkMenuItem *menuitem, gpointer user_data); /* in annot.c */
extern void quit_proc(GtkButton *button, gpointer user_data);                    /* in gtkwave.c */
extern void update_screen( );                                                    /* in gtkwave.c */
extern void clear_layer(gint i);                                                 /* in gtkwave.c */
extern void clear_all_layers( );                                                 /* in gtkwave.c */
extern void do_analysis(GtkButton *button, gpointer user_data);                  /* in analyze.c */

void prop_signals(GtkMenuItem *menuitem, gpointer user_data);
void prop_annotations(GtkMenuItem *menuitem, gpointer user_data);
void prop_wave(GtkMenuItem *menuitem, gpointer user_data);
gboolean on_canvas_configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer user_data);
gboolean on_canvas_expose_event(GtkWidget *widget, GdkEventExpose  *event, gpointer user_data);
void destroy_func(GtkObject *object, gpointer user_data);
void mode_button_clicked(GtkButton *button, gpointer user_data);
gboolean on_view_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
void resize(GtkWidget *widget, GtkRequisition  *requisition, gpointer user_data);
void print_setup_proc(GtkEditable *editable, gpointer user_data);
void show_find(GtkButton *button, gpointer user_data);
void hide_find(GtkButton *button, gpointer user_data);
void search_select(GtkWidget *widget, gpointer user_data);
void match_selected_annotations(GtkButton *button, gpointer user_data);
void dismiss_search_template(GtkButton *button, gpointer user_data);
gboolean on_find_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
gboolean on_search_template_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
void set_back(GtkButton *button, gpointer user_data);
void set_start(GtkWidget *widget, gpointer user_data);
void set_stop(GtkWidget *widget, gpointer user_data);
void set_ahead(GtkButton *button, gpointer user_data);
void set_signal(GtkEditable *editable, gpointer user_data);
void set_siglist(GtkEditable *editable, gpointer user_data);
void show_scope_window(GtkButton *button, gpointer user_data);
void show_command_window(GtkButton *button, gpointer user_data);
void edit_menu_file(GtkButton *button, gpointer user_data);
void recreate_analyze_popup(GtkButton *button, gpointer user_data);
void reload(GtkButton *button, gpointer user_data);
gboolean on_analyze_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
void dismiss_analyze_popup(GtkButton *button, gpointer user_data);
void reinitialize(GtkButton *button, gpointer user_data);
gboolean on_file_load_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
void ann_select(GtkWidget *widget, gpointer user_data);
void change_annotations(GtkButton *button, gpointer user_data);
void dismiss_ann_template(GtkButton *button, gpointer user_data);
void set_level_mode(GtkWidget *widget, gpointer user_data);
void dismiss_level_popup(GtkButton *button, gpointer user_data);
gboolean on_annotation_template_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
gboolean on_analysis_cmds_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
void dismiss_analysis_cmds(GtkButton *button, gpointer user_data);
void wfdbc_proc(GtkEditable *editable, gpointer user_data);
void wfdbp_proc(GtkEditable *editable, gpointer user_data);
void show_print(GtkButton *button, gpointer user_data);
void show_load(GtkMenuItem *menuitem, gpointer user_data);
void hide_load(GtkButton *button, gpointer user_data);
void save_proc(GtkMenuItem *menuitem, gpointer user_data);
void print_proc(GtkMenuItem *menuitem, gpointer user_data);
void show_print_setup(GtkMenuItem *menuitem, gpointer user_data);
void hide_print_setup(GtkButton *button, gpointer user_data);
gboolean on_print_setup_delete_event(GtkWidget *widget, GdkEvent*event, gpointer user_data);
void analyze_proc(GtkMenuItem *menuitem, gpointer user_data);
void show_log(GtkMenuItem *menuitem, gpointer user_data);
void show_mode(GtkMenuItem *menuitem, gpointer user_data);
void allow_editing(GtkMenuItem *menuitem, gpointer user_data);
void view_only(GtkMenuItem *menuitem, gpointer user_data);
void log_select(GtkWidget *widget, gpointer user_data);
gchar *get_wave_filename( gchar wave_file_type );
gboolean on_file_viewer_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
void dismiss_file_viewer( GtkButton *button, gpointer user_data);
void on_menu_quit_activate( GtkMenuItem *menuitem, gpointer user_data);

