/* file: helppan.c	        I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions for commandline help, and drawing the help pannel in Wave.

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include "wave.h"
#include "gwave.h"
#include "interface.h"
#include "support.h"

/* On-line help files for WAVE are located in *(helpdir)/wave.  Their names are
   of the form *(helpdir)/wave/TOPIC.hlp, where TOPIC is one of the topics
   given in the help popup menu (possibly abbreviated).  The master help file
   is *(helpdir)/wave/HELPFILE. */
#define HELPFILE	"wave.hlp"

void find_user_guide()
{
    FILE *ifile;

    sprintf(url, "%s/html/wug/wug.htm", wavedir->str);
    if ( (ifile = fopen(url, "r")) ) fclose(ifile);
    else if ( (ifile = fopen("wug.htm", "r")) ) {
	fclose(ifile);
	sprintf(url, "%s/wug.htm", getcwd(NULL, 256));
    }
    else
	strcpy(url, "http://www.physionet.org/physiotools/wug/");
}

void help()
{
    find_user_guide();
    g_printerr( "WAVE version %s (%s)\n %s", VERSION, __DATE__,
	    wfdberror());
    g_printerr( "usage: %s -r RECORD[+RECORD] [ options ]\n", pname->str);
    g_printerr( "\nOptions are:\n");
    g_printerr( " -a annotator-name  Open an annotation file\n");
    g_printerr(" -dpi XX[xYY]       Calibrate for XX [by YY] dots/inch\n");
    g_printerr( " -f TIME            Open the record beginning at TIME\n");
    g_printerr( " -g                 Use shades of grey only\n");
    g_printerr( " -H                 Use high-resolution mode\n");
    g_printerr( " -m                 Use black and white only\n");
    g_printerr( " -O                 Use overlay graphics\n");
    g_printerr( " -p PATH            Search for input files in PATH\n");
    g_printerr( "                     (if not found in $WFDB)\n");
    g_printerr( " -s SIGNAL [SIGNAL ...]  Initialize the signal list\n");
    g_printerr( " -S                 Use a shared colormap\n");
    g_printerr( " -Vx                Set initial display option x\n");
    if (g_getenv("DISPLAY") == NULL) {
	g_printerr( "\n%s is an X11 client.  ", pname->str);
	g_printerr( "You must specify the X server\n");
	g_printerr( "connection for it in the DISPLAY environment ");
	g_printerr( "variable.\n");
    }
    if (g_getenv("WFDB") == NULL) {
	g_printerr( "\nBe sure to set the WFDB environment variable to\n");
	g_printerr( "indicate a list of locations that contain\n");
	g_printerr( "input files for %s.\n", pname->str);
    }
    g_printerr( "\nFor more information, type `more %s/wave/%s',\n",
	    wavedir->str, HELPFILE);
    g_printerr( "or open `%s' using your web browser.\n", url);
    exit(1);
}
