/* file: modepan.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions for the mode (aka view) panel of WAVE

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include "wave.h"
#include "gwave.h"
#include "interface.h"
#include "support.h"

#define SHOW_SUBTYPE 0
#define SHOW_CHAN 1   
#define SHOW_NUM 2    
#define SHOW_AUX 3    
#define SHOW_MARKER 4 
#define SHOW_SIGNAME 5
#define SHOW_BASELINE 6
#define SHOW_LEVEL 7  

#define UNDO_BUTTON 0
#define REDRAW_BUTTON 1
#define SAVE_BUTTON 2
#define DISMISS_BUTTON 3

GtkWidget *grid_item, *opt_item[8], *sig_item, *ann_item, *ov_item, *tim_item, 
  *ts_item, *vs_item, *redraw_item;

GtkWidget *mode_frame;

/* Undo any mode changes. */
void
mode_undo()
{
  gtk_option_menu_set_history( GTK_OPTION_MENU( ts_item ), tsa_index);
  gtk_option_menu_set_history( GTK_OPTION_MENU( vs_item ), vsa_index);
  gtk_option_menu_set_history( GTK_OPTION_MENU( sig_item ), sig_mode);
  gtk_option_menu_set_history( GTK_OPTION_MENU( ann_item ), ann_mode);
  gtk_option_menu_set_history( GTK_OPTION_MENU( ov_item ), overlap);
  gtk_option_menu_set_history( GTK_OPTION_MENU( tim_item ), time_mode);
  gtk_option_menu_set_history( GTK_OPTION_MENU( grid_item ), grid_mode);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON ( opt_item[SHOW_SUBTYPE] ), show_subtype);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON ( opt_item[SHOW_CHAN] ), show_chan);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON ( opt_item[SHOW_NUM] ), show_num);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON ( opt_item[SHOW_AUX] ), show_aux);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON ( opt_item[SHOW_MARKER] ), show_marker);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON ( opt_item[SHOW_SIGNAME] ), show_signame);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON ( opt_item[SHOW_BASELINE] ), show_baseline);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON ( opt_item[SHOW_LEVEL] ), show_level);
}

/* Redraw and take down the window. */
void
redraw_proc()
{
  disp_proc(NULL, GINT_TO_POINTER((gint)'.'));
}

int mode_popup_active = -1;

void create_mode_popup()
{
  
  mode_frame = create_view ();    
  
  opt_item[SHOW_SUBTYPE] = lookup_widget(GTK_WIDGET(mode_frame), "subtype");
  opt_item[SHOW_CHAN] = lookup_widget(GTK_WIDGET(mode_frame), "chan_field");
  opt_item[SHOW_NUM] = lookup_widget(GTK_WIDGET(mode_frame), "num_field");  
  opt_item[SHOW_AUX] = lookup_widget(GTK_WIDGET(mode_frame), "aux_field");  
  opt_item[SHOW_MARKER] = lookup_widget(GTK_WIDGET(mode_frame), "markers");
  opt_item[SHOW_SIGNAME] = lookup_widget(GTK_WIDGET(mode_frame), "signal_names");
  opt_item[SHOW_BASELINE] = lookup_widget(GTK_WIDGET(mode_frame), "baseline");
  opt_item[SHOW_LEVEL] = lookup_widget(GTK_WIDGET(mode_frame), "level");
  
  ts_item = lookup_widget(GTK_WIDGET(mode_frame), "time_scale_option");
  vs_item = lookup_widget(GTK_WIDGET(mode_frame), "amplitude_scale_option");
  sig_item = lookup_widget(GTK_WIDGET(mode_frame), "draw_option");
  grid_item = lookup_widget(GTK_WIDGET(mode_frame), "grid_option");
  ann_item = lookup_widget(GTK_WIDGET(mode_frame), "show_annotations_option");
  ov_item = lookup_widget(GTK_WIDGET(mode_frame), "overlap_option");
  tim_item = lookup_widget(GTK_WIDGET(mode_frame), "time_display_option");
  
  redraw_item = lookup_widget(GTK_WIDGET(mode_frame), "redraw");
  
  mode_popup_active = 0;
}

void show_mode(GtkMenuItem *menuitem, gpointer user_data)
{
  if (mode_popup_active < 0) create_mode_popup();
  gtk_widget_show (mode_frame);
  mode_popup_active = 1;
}

void set_modes()
{
  int i, otsai = tsa_index, ovsai = vsa_index;
  double osh = canvas_height_mv, osw = canvas_width_sec;
  static double vsa[] = { 1.0, 2.5, 5.0, 10.0, 20.0, 40.0, 100.0 };

  /* If the panel has never been instantiated, there is nothing to do. */
  if (mode_popup_active < 0) return;


  /* Read the current panel settings, beginning with the grid option. */
  switch ( (grid_mode = get_option_menu_active_index( GTK_OPTION_MENU(grid_item) )) ) {
     case 0: ghflag = gvflag = visible = 0; break;
     case 1: ghflag = 0; gvflag = visible = 1; break;
     case 2: ghflag = visible = 1; gvflag = 0; break;
     case 3: ghflag = gvflag = visible = 1; break;
     case 4: ghflag = gvflag = visible = 2; break;
     case 5: ghflag = visible = 1; gvflag = 3; break;
     case 6: ghflag = visible = 2; gvflag = 3; break;
  }

  /* Next, check the annotation display options.  */
  show_subtype =  gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(opt_item[SHOW_SUBTYPE]) );
  show_chan = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(opt_item[SHOW_CHAN]) );
  show_num = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(opt_item[SHOW_NUM]) );
  show_aux = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(opt_item[SHOW_AUX]) );
  show_marker = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(opt_item[SHOW_MARKER]) );
  show_signame = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(opt_item[SHOW_SIGNAME]) );
  show_baseline = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(opt_item[SHOW_BASELINE]) );
  show_level = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(opt_item[SHOW_LEVEL]) );
  
  /* Check the signal display options next. */
  i = sig_mode;

  sig_mode = get_option_menu_active_index( GTK_OPTION_MENU(sig_item) );
  if (i != sig_mode)
    set_baselines();
  
  /* Check the annotation display mode next. */
  ann_mode = get_option_menu_active_index( GTK_OPTION_MENU(ann_item) );  
  overlap = get_option_menu_active_index( GTK_OPTION_MENU(ov_item) );

  /* Check the time display mode next. */
  i = time_mode;
  time_mode =  get_option_menu_active_index( GTK_OPTION_MENU(tim_item) );
  
  /* The `nsig > 0' test is a bit of a kludge -- we don't want to reset the
     time_mode before the record has been opened, because we don't know if
     absolute times are available until then. */
  if (nsig > 0 && time_mode == 1)
    (void)wtimstr(0L);	/* check if absolute times are available --
  			   if not, time_mode is reset to 0 */
  if (i != time_mode) {
    set_start_time(wtimstr(display_start_time));
    set_end_time(wtimstr(display_start_time + nsamp));
    reset_start();
    reset_stop();
  }

  /* The purpose of the complex method of computing canvas_width_sec is to
     obtain a "rational" value for it even if the frame has been resized.
     First, we determine the number of 5 mm time-grid units in the window
     (note that the resize procedure in xvwave.c guarantees that this will
     be an integer;  the odd calculation is intended to take care of
     roundoff error in pixel-to-millimeter conversion).  For each scale,
     the multiplier of u is simply the number of seconds that would be
     represented by 5 mm.   Since u is usually a multiple of 5 (except on
     small displays, or if the frame has been resized to a small size),
     the calculated widths in seconds are usually integers, at worst
     expressible as an integral number of tenths of seconds. */
  if ((i = get_option_menu_active_index( GTK_OPTION_MENU(ts_item) )) >= 0 && i < 13) {
    int u = ((int)(canvas_width/dmmx(1) + 1)/5);	/* number of 5 mm
  							   time-grid units */
    switch (tsa_index = i) {
    case 0:	/* 0.25 mm/min */
      mmpersec = (0.25/60.);
      canvas_width_sec = 1200 * u; break;
    case 1:	/* 1 mm/min */
      mmpersec = (1./60.);
      canvas_width_sec = 300 * u; break;
    case 2:	/* 5 mm/min */
      mmpersec = (5./60.);
      canvas_width_sec = 60 * u; break;
    case 3:	/* 25 mm/min */
      mmpersec = (25./60.);
      canvas_width_sec = 12 * u; break;
    case 4:	/* 50 mm/min */
      mmpersec = (50./60.);
      canvas_width_sec = 6 * u; break;
    case 5:	/* 125 mm/min */
      mmpersec = (125./60.);
      canvas_width_sec = (12 * u) / 5; break;
    case 6:	/* 250 mm/min */
      mmpersec = (250./60.);
      canvas_width_sec = (6 * u) / 5; break;
    case 7:	/* 500 mm/min */
      mmpersec = (500./60.);
      canvas_width_sec = (3 * u) / 5; break;
     case 8:	/* 12.5 mm/sec */
      mmpersec = 12.5;
      canvas_width_sec = (2 * u) / 5; break;
    case 9:	/* 25 mm/sec */
      mmpersec = 25.;
      canvas_width_sec = u / 5; break;
    case 10:	/* 50 mm/sec */
      mmpersec = 50.;
      canvas_width_sec = u / 10; break;
    case 11:	/* 125 mm/sec */
      mmpersec = 125.;
      canvas_width_sec = u / 25; break;
    case 12:	/* 250 mm/sec */
      mmpersec = 250.;
      canvas_width_sec = u / 50; break;
    }
  }

  /* Computation of canvas_height_mv could be as complex as above, but
     it doesn't seem so important to obtain a "rational" value here. */
  if ((i = get_option_menu_active_index( GTK_OPTION_MENU(vs_item) )) >= 0 && i < 7) {
    mmpermv = vsa[i];
    canvas_height_mv = canvas_height/dmmy(vsa[vsa_index = i]);
  }
  if (osh != canvas_height_mv || osw != canvas_width_sec ||
      otsai != tsa_index || ovsai != vsa_index) {
    vscale[0] = 0.0;
    calibrate();
  }
}

/* make the popup disappear. */
void dismiss_mode()
{
  /* If the panel is currently visible, make it go away. */
  if (mode_popup_active > 0 ) {
    gtk_widget_hide(mode_frame);
    mode_popup_active = 0;
  }
}

/* Time-to-string conversion functions.  These functions use those in the
   WFDB library, but ensure that (1) elapsed times are displayed if time_mode
   is 0, and (2) absolute times (if available) are displayed without brackets
   if time_mode is non-zero. */
long wstrtim(s)
char *s;
{
  long t;

  while (*s == ' ' || *s == '\t') s++;
  if (time_mode == 1 && *s != '[' && *s != 's' && *s != 'c' && *s != 'e') {
    char buf[80];
    
    sprintf(buf, "[%s]", s);
    s = buf;
  }
  t = strtim(s);
  if (*s == '[') {	/* absolute time specified - strtim returns a negated
			   sample number if s is valid */
    if (t > 0L) t = 0L;	/* invalid s (before sample 0) -- reset t */
    else t = -t;	/* valid s -- convert t to a positive sample number */
  }
  return (t);
}

char *wtimstr(t)
long t;
{
  switch (time_mode) {
  case 0:
  default:
    if (t == 0L) return ("0:00");
    else if (t < 0L) t = -t;
    return (timstr(t));
  case 1:
    {
      char *p, *q;
      
      if (t > 0L) t = -t;
      p = timstr(t);
      if (*p == '[') {
	p++;
	q = p + strlen(p) - 1;
	if (*q == ']') *q = '\0';
      }
      else {
	time_mode = 0;
	if (tim_item) gtk_option_menu_set_history(GTK_OPTION_MENU(tim_item), time_mode);
      }
      return (p);
    }
  case 2:
    {
      static char buf[12];
      
      if (t < 0L) t = -t;
      sprintf(buf, "s%ld", t);
      return (buf);
    }
  }
}

char *wmstimstr(t)
     long t;
{
  switch (time_mode) {
  case 0:
  default:
    if (t == 0L) return ("0:00");
    else if (t < 0L) t = -t;
    return (mstimstr(t));
  case 1:
    {
      char *p, *q;
      
      if (t > 0L) t = -t;
      p = mstimstr(t);
      if (*p == '[') {
	p++;
	q = p + strlen(p) - 1;
	if (*q == ']') *q = '\0';
      }
      else {
	time_mode = 0;
	if (tim_item) gtk_option_menu_set_history(GTK_OPTION_MENU(tim_item), time_mode);
      }
      return (p);
    }
  case 2:
    {
      static char buf[12];
      
      if (t < 0L) t = -t;
      sprintf(buf, "s%ld", t);
      return (buf);
    }
  }
}


void
mode_button_clicked  (GtkButton       *button,
                      gpointer         user_data)
{
  switch ( GPOINTER_TO_INT(user_data) ) {
  case UNDO_BUTTON:
    mode_undo();
    break;
  case REDRAW_BUTTON:
    redraw_proc();
    break;
  case SAVE_BUTTON:
    redraw_proc();
    save_defaults();
    break;
  case DISMISS_BUTTON:
    dismiss_mode();
  }
}


gboolean
on_view_delete_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  /* don't kill window, instead divert to dismiss button */
  mode_button_clicked(NULL, GINT_TO_POINTER(DISMISS_BUTTON));
  return TRUE;
}


