/* file: notice.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Generic functions for notice dialogs for Wave.

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include "wave.h"
#include "gwave.h"
#include "interface.h"
#include "support.h"

gint notice_yes_clicked( GtkWidget * widget, gpointer data )
{
  *(int *)(data) = NOTICE_YES;
  gtk_main_quit();
  return (TRUE);
}

gint notice_no_clicked( GtkWidget * widget, gpointer data )
{
  *(int *)(data) = NOTICE_NO;
  gtk_main_quit();
  return (TRUE);
}

GtkWidget *
create_notice ( gchar *notice_string, gchar *yes_button_string, 
		gchar *no_button_string, gint *result ) {

  GtkWidget *notice_primitive;
  GtkWidget *notice_message_label;
  GtkWidget *notice_hbuttonbox;
  GtkWidget *notice_yes_button = NULL;
  GtkWidget *notice_no_button =NULL;


  notice_primitive = create_notice_primitive();

  gtk_signal_connect( GTK_OBJECT( notice_primitive ), "delete_event",
		      GTK_SIGNAL_FUNC ( notice_no_clicked ), result );
  

  notice_message_label = lookup_widget ( GTK_WIDGET(notice_primitive), "notice_message_label" );
  notice_hbuttonbox = lookup_widget ( GTK_WIDGET(notice_primitive), "notice_hbuttonbox");

  gtk_label_set_text(GTK_LABEL(notice_message_label), notice_string);
  
  if ( yes_button_string != NULL ) {
    notice_yes_button = gtk_button_new_with_label(yes_button_string);
    gtk_box_pack_start( GTK_BOX( notice_hbuttonbox ), notice_yes_button, TRUE, FALSE, 0);
    gtk_signal_connect( GTK_OBJECT( notice_yes_button ), "clicked",
			GTK_SIGNAL_FUNC ( notice_yes_clicked ), result );
    gtk_widget_show( notice_yes_button );
  }
  
  if ( no_button_string != NULL ) {
    notice_no_button = gtk_button_new_with_label(no_button_string);
    gtk_box_pack_start( GTK_BOX( notice_hbuttonbox ), notice_no_button, TRUE, FALSE, 0);
    gtk_signal_connect( GTK_OBJECT( notice_no_button ), "clicked",
			GTK_SIGNAL_FUNC ( notice_no_clicked ), result );
    gtk_widget_show( notice_no_button );
  }
  
  return notice_primitive;
}

int notice_prompt(char *notice_string, char *yes_button_string, 
		  char *no_button_string )
{
  GtkWidget *notice;
  int result;
  
  notice = create_notice( notice_string, yes_button_string, no_button_string, &result);
  gtk_widget_show ( notice );
  
  gtk_main();
  
  gtk_widget_destroy( GTK_WIDGET(notice) );
  
  return result;
}



