/* file: search.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions for the search panel of WAVE

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include <gtk/gtk.h>
#include "wave.h"
#include "gwave.h"
#include "interface.h"
#include "support.h"

GtkWidget *search_frame;
GtkWidget *s_anntyp_item, *s_subtyp_item, *s_num_item, *s_chan_item, *s_aux_item;
GtkWidget *s_anntyp_mask, *s_subtyp_mask, *s_num_mask, *s_chan_mask, *s_aux_mask;

int search_popup_active = -1;

/* Make the search template popup window disappear. */
void dismiss_search_template(GtkButton *button, gpointer user_data)
{
  if (search_popup_active > 0) {
    gtk_widget_hide(search_frame);
    search_popup_active = 0;
  }
}

/* Update the search template following a selection in the popup window. */
void search_select(GtkWidget *widget, gpointer user_data)
{
  int a;
  static char tmp_aux[257];
  

  switch ((a = GPOINTER_TO_INT(user_data))) {
    case 'T':
      if ( get_menu_active_index(GTK_MENU(widget)) == 1)
	search_mask |= M_ANNTYP;	/* set anntyp mask bit */
      else
	search_mask &= ~M_ANNTYP;	/* clear anntyp mask bit */
      break;
    case 'S':
      if (get_menu_active_index(GTK_MENU(widget)) == 1)
	search_mask |= M_SUBTYP;	/* set subtyp mask bit */
      else
      search_mask &= ~M_SUBTYP;	/* clear subtyp mask bit */
      break;
    case 'N':
      if (get_menu_active_index(GTK_MENU(widget)) == 1)
	search_mask |= M_NUM;	/* set num mask bit */
      else
	search_mask &= ~M_NUM;	/* clear num mask bit */
      break;
    case 'C':
      if (get_menu_active_index(GTK_MENU(widget)) == 1)
	search_mask |= M_CHAN;	/* set chan mask bit */
      else
	  search_mask &= ~M_CHAN;	/* clear chan mask bit */
      break;
    case 'A':
      if (get_menu_active_index(GTK_MENU(widget)) == 1)
	search_mask |= M_AUX;	/* set aux mask bit */
      else
	search_mask &= ~M_AUX;	/* clear aux mask bit */
      break;
    case 't': 
      search_template.anntyp = get_menu_active_index(GTK_MENU(widget));
      break;
    case 's': 
      search_template.subtyp = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
      break;
    case 'n': 
      search_template.num = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
      break;
    case 'c': 
      search_template.chan = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
      break;
    case 'a': 
      strcpy(tmp_aux+1, gtk_entry_get_text(GTK_ENTRY(widget)));
      if ( (tmp_aux[0] = strlen(tmp_aux+1)) )
	search_template.aux = tmp_aux;
      else
	search_template.aux = NULL;
      break;
  }
}

static char *mstr[53];

static void s_create_mstr_array()
{
  char *dp, mbuf[5], *mp;
  int i;
  unsigned int l, lm;
  
  mstr[0] = ".    (Deleted annotation)";
  for (i = 1; i <= ACMAX; i++) {
    if ((mp = annstr(i)) == NULL) {
      sprintf(mbuf, "[%d]", i);
      mp = mbuf;
    }
    if ((dp = anndesc(i)) == NULL)
      dp = "(unassigned annotation type)";
    lm = strlen(mp);
    l = strlen(dp) + 6;
    if (lm > 4) l += lm - 4;
    if ((mstr[i] = (char *)malloc(l)) == NULL)
      break;
    strcpy(mstr[i], mp);
    do {
      mstr[i][lm] = ' ';
    } while (++lm < 5);
    strcpy(mstr[i]+lm, dp);
  }
  mstr[ACMAX+1] = ":    (Index mark)";
  mstr[ACMAX+2] = "<    (Start of analysis)";
  mstr[ACMAX+3] = ">    (End of analysis)";
}

/* Set the annotation template window `Type' item. */
void
match_selected_annotations(GtkButton *button,
			   gpointer user_data)
{
    if (attached == NULL) return;
    gtk_option_menu_set_history( GTK_OPTION_MENU(s_anntyp_item), attached->this.anntyp); 
    gtk_entry_set_text( GTK_ENTRY(s_aux_item),
			attached->this.aux ? attached->this.aux+1 : ""); 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(s_subtyp_item), attached->this.subtyp);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(s_chan_item), attached->this.chan);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(s_num_item), attached->this.num);
    gtk_option_menu_set_history( GTK_OPTION_MENU(s_anntyp_mask), 1);
    gtk_option_menu_set_history( GTK_OPTION_MENU(s_aux_mask), 1);
    gtk_option_menu_set_history( GTK_OPTION_MENU(s_subtyp_mask), 1);
    gtk_option_menu_set_history( GTK_OPTION_MENU(s_chan_mask), 1);
    gtk_option_menu_set_history( GTK_OPTION_MENU(s_num_mask), 1);
    search_template = attached->this;
    search_mask = M_ANNTYP | M_SUBTYP | M_CHAN | M_NUM;
    if (search_template.aux) search_mask |= M_AUX;
}

/* Set up popup window for defining the search template. */
static void create_search_template_popup()
{
  gint i;
  GtkWidget *menu, *menuitem;

  s_create_mstr_array();
  search_template.anntyp = 1;
  
  search_frame = create_search_template();  
  s_anntyp_mask = lookup_widget( GTK_WIDGET(search_frame), "type_mask_option");
  gtk_signal_connect( GTK_OBJECT (gtk_option_menu_get_menu( GTK_OPTION_MENU (s_anntyp_mask))), 
		      "selection-done", GTK_SIGNAL_FUNC(search_select), GINT_TO_POINTER((gint)'T'));

  s_anntyp_item = lookup_widget( GTK_WIDGET(search_frame), "type_option");
  menu = gtk_menu_new();
  for (i=0; i<54; i++) {
    menuitem = gtk_radio_menu_item_new_with_label(NULL, mstr[i]);
    gtk_menu_append( GTK_MENU( menu ), menuitem);
    gtk_widget_show(menuitem);
  }
  gtk_signal_connect( GTK_OBJECT (menu), "selection-done",
		      GTK_SIGNAL_FUNC(search_select), GINT_TO_POINTER((gint)'t'));
  gtk_option_menu_remove_menu( GTK_OPTION_MENU (s_anntyp_item) );
  gtk_option_menu_set_menu( GTK_OPTION_MENU (s_anntyp_item), menu );
  gtk_option_menu_set_history( GTK_OPTION_MENU( s_anntyp_item ), 1 );

  s_aux_mask = lookup_widget( GTK_WIDGET (search_frame), "aux_mask_option");
  gtk_signal_connect( GTK_OBJECT (gtk_option_menu_get_menu( GTK_OPTION_MENU (s_aux_mask) )),
		      "selection-done", GTK_SIGNAL_FUNC(search_select), GINT_TO_POINTER((gint)'A'));
  s_aux_item = lookup_widget( GTK_WIDGET (search_frame), "aux_entry");

  s_subtyp_mask = lookup_widget( GTK_WIDGET( search_frame ), "subtype_mask_option");
  gtk_signal_connect( GTK_OBJECT (gtk_option_menu_get_menu( GTK_OPTION_MENU (s_subtyp_mask))),
		      "selection-done", GTK_SIGNAL_FUNC(search_select), GINT_TO_POINTER((gint)'S'));
  s_subtyp_item = lookup_widget( GTK_WIDGET( search_frame ), "subtype_spinbutton");

  s_chan_mask = lookup_widget( GTK_WIDGET( search_frame ), "chan_mask_option");
  gtk_signal_connect( GTK_OBJECT (gtk_option_menu_get_menu( GTK_OPTION_MENU (s_chan_mask))),
		      "selection-done", GTK_SIGNAL_FUNC(search_select), GINT_TO_POINTER((gint)'C'));
  s_chan_item = lookup_widget( GTK_WIDGET( search_frame ), "chan_spinbutton");

  s_num_mask = lookup_widget( GTK_WIDGET( search_frame ), "num_mask_option");
  gtk_signal_connect( GTK_OBJECT (gtk_option_menu_get_menu( GTK_OPTION_MENU (s_num_mask))),
		      "selection-done", GTK_SIGNAL_FUNC(search_select), GINT_TO_POINTER((gint)'N'));
  s_num_item = lookup_widget( GTK_WIDGET( search_frame ), "num_spinbutton");

}

/* Make the search template popup window appear. */

void
show_search_template                   (GtkButton       *button,
                                        gpointer         user_data)
{
    if (search_popup_active < 0) create_search_template_popup();    
    gtk_widget_show( search_frame );
    search_popup_active = 1;
    set_find_item("");
}


gboolean
on_search_template_delete_event        (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  dismiss_search_template(NULL, NULL);
  return TRUE;
}

